<?php
/*
    Copyright (c) 2002-2007 Netlor
    Copyright (c) 2007-2008 Ovensia
    Contributors hold Copyright (c) to their code submissions.

    This file is part of Ploopi.

    Ploopi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ploopi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ploopi; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Gestion de l'accs aux donnes.
 *
 * @package ploopi
 * @subpackage database
 * @copyright Netlor, Ovensia
 * @license GNU General Public License (GPL)
 * @author Stphane Escaich
 */

/**
 * Classe gnrique d'accs aux donnes.
 * Permet la manipulation d'enregistrements de base de donnes sous forme d'objets.
 *
 * @package ploopi
 * @subpackage database
 * @copyright Netlor, Ovensia
 * @license GNU General Public License (GPL)
 * @author Stphane Escaich
 */

class data_object
{
    /**
     * Nom de la classe
     *
     * @var string
     */

    private $classname;

    /**
     * Nom de la table
     *
     * @var string
     */

    private $tablename;

    /**
     * Nom de la table (intgration SQL)
     *
     * @var string
     */

    private $tablename_quoted;

    /**
     * Tableau index des champs qui composent la cl primaire
     *
     * @var array
     */

    private $idfields;

    /**
     * Tableau associatif des valeurs qui composent la cl primaire
     *
     * @var array
     */

    private $id;

    /**
     * Objet de connexion  la base de donnes
     *
     * @var ploopi_db
     * @see ploopi_db
     */

    private $db;

    /**
     * Connexion  la base de donnes
     *
     * @var resource
     * @see data_objet::setdb
     */

    private $resultid;

    /**
     * Nombre de ligne du dernier rsultat
     *
     * @var int
     */

    private $numrows;

    /**
     * Requte SQL gnre par l
     *
     * @var string
     * @see data_objet::getsql
     */

    private $sql;

    /**
     * Contenu d'un enregistrement de la table dans un tableau associatif : champ => valeur
     *
     * @var array
     */

    public $fields;

    /**
     * Indique s'il s'agit d'un nouvel enregistrement (true) ou d'un enregistrement existant (false)
     *
     * @var boolean
     */

    public $new;

    /**
     * Constructeur de la classe
     *
     * @param string nom de la table
     * @param string champ cl n1
     * @param string champ cl n2
     * @param string champ cl nX
     *
     * @return data_object
     *
     */

    public function data_object()
    {
        global $db;
        // arg(0) : tablename

        $numargs = func_num_args();
        $this->classname = get_class($this) ;
        $this->tablename = func_get_arg(0);
        $this->tablename_quoted = '`'.str_replace('.', '`.`', $this->tablename).'`';
        $this->idfields = array();
        $this->id = array();
        $this->fields = array();

        if ($numargs == 1) // cas particulier n1, pas de cl dfinie, on prend id par dfaut
        {
            $this->idfields[0] = 'id';
            $this->fields['id'] = null;
        }
        else
        {
            if ($numargs == 2 && is_array($keys = func_get_arg(1))) // cas particulier n2, les cls sont dfinies dans un tableau (nouvelle mthode)
            {
                if (sizeof($keys) == 0) // cas particulier n2b, pas de cl dfinie, on prend id par dfaut
                {
                    $this->idfields[0] = 'id';
                    $this->fields['id'] = null;
                }
                else
                {
                    $i = 0;
                    foreach(func_get_arg(1) as $key)
                    {
                        $this->idfields[$i] = $key;
                        $this->fields[$this->idfields[$i]] = null;
                        $this->id[$this->idfields[$i]] = null;
                        $i++;
                    }
                }
            }
            else // cas gnral, les cls sont dfinies dans une liste d'arguments
            {
                for ($i = 1; $i < $numargs; $i++)
                {
                    if (!is_null(func_get_arg($i)))
                    {
                        $this->idfields[$i-1] = func_get_arg($i);
                        $this->fields[$this->idfields[$i-1]] = null;
                        $this->id[$this->idfields[$i-1]] = null;
                    }
                }
            }
        }

        $this->db = &$db;

        $this->new = true;
    }


    /**
     * Permet de modifier la connexion  la base de donnes
     *
     * @param ressource $db objet de connexion  la base de donnes
     */

    public function setdb($db)
    {
        unset($this->db);
        $this->db = $db;
    }


    /**
     * Permet de mettre  jour les proprits de l'objet (les champs de la table)
     *
     * @param array $values tableau associatif contenant les valeurs tel que "prefixe_nomduchamp" => "valeur"
     * @param string $prefix prfixe utilis
     */

    public function setvalues($values, $prefix = '')
    {
        // par dfaut on rcupre les champs du formulaire ($values)
        $lprefix = strlen($prefix);
        foreach ($values as $key => $value)
        {
            if ($lprefix == 0) $this->fields[$key] = $value;
            else
            {
                $pref = substr($key, 0, $lprefix);
                if ($pref == $prefix)
                {
                    $prop = substr($key, $lprefix);
                    $this->fields[$prop] = $value;
                }
            }
        }

    }

    /**
     * Ouvre un enregistrement de la table et met  jour l'objet
     *
     * @param mixed valeur du champ 1 de la cl
     * @param mixed valeur du champ 2 de la cl
     * @param mixed valeur du champ X de la cl
     *
     * @return int nombre d'enregistrements
     */

    public function open() // id0, id1, id2, etc...
    {
        $args = func_get_args();

        if(sizeof($args) == 1 && is_array($args[0])) $args = $args[0];

        $numargs = sizeof($args);

        if ($numargs == sizeof($this->idfields))
        {
            for ($i = 0; $i < $numargs; $i++) $id[$i] = $args[$i];

            $this->sql = "SELECT * FROM {$this->tablename_quoted} WHERE `{$this->idfields[0]}` = '".$this->db->addslashes($id[0])."'";

            for ($i = 1; $i < $numargs; $i++) $this->sql .= " AND `{$this->idfields[$i]}` = '".$this->db->addslashes($id[$i])."'";

            $this->resultid = $this->db->query($this->sql);
            $this->numrows = $this->db->numrows($this->resultid);
            $this->fields = $this->db->fetchrow($this->resultid);

            for ($i = 0; $i < $numargs; $i++) $this->id[$this->idfields[$i]] = $this->fields[$this->idfields[$i]] = $id[$i];

            if ($this->numrows > 0) $this->new = false;

            return $this->numrows > 0;
        }
        else return false;

    }

    /**
     * Mthode d'ouverture spciale pour "convertir" une ligne de recordset en objet
     *
     * @param array $row ligne de recordset
     */

    public function open_row($row)
    {
        $this->fields = $row;

        foreach($this->idfields as $field) $this->id[$field] = $row[$field];

        $this->new = false;
    }

    /**
     * Insre ou met  jour l'enregistrement dans la base de donnes
     *
     * @return mixed valeur de la cl primaire
     */

    public function save()
    {

        if ($this->new) // insert
        {
            $listvalues='';

            // build insert
            $arrValues = array();
            foreach ($this->fields as $key => $value)
            {
                $arrValues[] = (is_null($value)) ? "{$this->tablename_quoted}.`{$key}` = null" : "{$this->tablename_quoted}.`{$key}` = '".$this->db->addslashes($value)."'";
            }

            $listvalues = (empty($arrValues)) ? '' : 'SET '.implode(', ', $arrValues);

            $this->sql = "INSERT INTO {$this->tablename_quoted} {$listvalues}"; // construction de la requte
            $this->db->query($this->sql);

            // get "static" key
            foreach($this->idfields as $fieldname) if (isset($this->fields[$fieldname])) $this->id[$fieldname] = $this->fields[$fieldname];

            // get insert id from insert (if 1 field primary key and autokey)
            if (sizeof($this->idfields) >= 1 && $this->db->insertid() !== 0)
            {
                $this->id[$this->idfields[0]] = $this->fields[$this->idfields[0]] = $this->db->insertid();
            }

            $this->new = false;
        }
        else // update
        {
            $listvalues='';
            $arrValues = array();
            foreach ($this->fields as $key => $value)
            {
                $arrValues[] = (is_null($value)) ? "{$this->tablename_quoted}.`{$key}` = null" : "{$this->tablename_quoted}.`{$key}` = '".$this->db->addslashes($value)."'";
            }

            $listvalues = (empty($arrValues)) ? '' : implode(', ', $arrValues);

            // build request
            $this->sql = "UPDATE {$this->tablename_quoted} SET {$listvalues} WHERE {$this->tablename_quoted}.`{$this->idfields[0]}` = '".$this->db->addslashes($this->id[$this->idfields[0]])."'";
            for ($i = 1; $i < sizeof($this->idfields); $i++) $this->sql .= " AND {$this->tablename_quoted}.`{$this->idfields[$i]}` = '".$this->db->addslashes($this->id[$this->idfields[$i]])."'";

            $this->db->query($this->sql);
        }

        // return key (array if multiple key)
        if (sizeof($this->idfields) == 1) return ($this->fields[$this->idfields[0]]);
        else
        {
            $res = array();
            foreach($this->idfields as $idfield) $res[] = $this->fields[$idfield];
            return ($res);
        }
    }

    /**
     * Supprime l'enregistrement dans la base de donnes
     */

    public function delete()
    {
        $numargs = func_num_args();
        if ($numargs > 0) for ($i = 0; $i < $numargs; $i++) $this->fields[$this->idfields[$i]] = func_get_arg($i);

        $this->sql = "DELETE FROM {$this->tablename_quoted} WHERE {$this->tablename_quoted}.`{$this->idfields[0]}` = '".$this->db->addslashes($this->fields[$this->idfields[0]])."'";
        for ($i = 1; $i < sizeof($this->idfields); $i++) $this->sql .= " AND {$this->tablename_quoted}.`{$this->idfields[$i]}` = '".$this->db->addslashes($this->fields[$this->idfields[$i]])."'";

        $this->db->query($this->sql);
    }

    /**
     * Initialise les proprits de l'objet avec la structure de la table
     */

    public function init_description()
    {
        $this->sql = "DESCRIBE {$this->tablename_quoted}";
        $result = $this->db->query($this->sql);
        while ($fields = $this->db->fetchrow($result)) $this->fields[$fields['Field']] = '';
    }

    /**
     * Met  jour les proprits id_user, id_workspace, id_module de l'objet avec le contenu de la session
     */

    public function setuwm()
    {
        if (!isset($_SESSION['ploopi'])) return false;

        $this->fields['id_user'] = $_SESSION['ploopi']['userid'] ;
        $this->fields['id_workspace'] = $_SESSION['ploopi']['workspaceid'];
        $this->fields['id_module'] = $_SESSION['ploopi']['moduleid'];
    }

    /**
     * Gnre un dump SQL de l'enregistrement
     *
     * @return string dump SQL
     */

    public function dump()
    {
        $listvalues='';

        // build insert
        foreach ($this->fields as $key => $value)
        {
            if ($key != '' && !is_null($value))
            {
                if ($listvalues != '') $listvalues .= ', ';
                $listvalues .= "{$this->tablename_quoted}.`{$key}` = '".$this->db->addslashes($value)."'";
            }
        }

        return ("INSERT INTO {$this->tablename_quoted} SET {$listvalues}");
    }

    /**
     * Gnre des variables templates  partir des proprits de l'objet
     *
     * @param Template $tpl template
     * @param string $prefix prfixe  ajouter (optionnel)
     */

    public function totemplate(&$tpl, $prefix = '')
    {
        $array_vars = array();
        foreach($this->fields as $key => $value) $array_vars[strtoupper("{$prefix}{$key}")] = $value;
        $tpl->assign_vars($array_vars);
    }

    /**
     * Retourne le script SQL de cration de la table
     *
     * @return string script SQL
     */

    public function getsqlstructure()
    {
        $sql = "CREATE TABLE {$this->tablename_quoted} (";

        $fields = '';

        foreach ($this->fields as $key => $value)
        {
            if ($key != '')
            {
                if ($fields != '') $fields .= ",\n";
                $fields .= "`{$key}` varchar(255) NOT NULL default ''";
            }
        }

        $sql .= $fields.") TYPE=MyISAM;";

        return($sql);
    }

    /**
     * Retourne la dernire requte SQL excute
     *
     * @return string
     */

    public function getsql() { return $this->sql; }

    /**
     * Retourne la table associe  l'instance
     *
     * @return string nom de la table
     */

    public function gettablename() { return $this->tablename; }

    /**
     * Retourne true si l'enregistrement n'existe pas encore dans la base de donnes
     *
     * @return boolean
     */

    public function isnew() { return $this->new; }

    /**
     * Retourne un hash de la cl de l'enregistrement
     *
     * @return string
     */
    public function gethash()
    {
        $arrHash = array();
        foreach($this->idfields as $fieldname) if (isset($this->id[$fieldname])) $arrHash[] = $this->id[$fieldname];
        return(implode(',', $arrHash));
    }

    /**
     * Retourne la collection
     *
     * @return data_object_collection
     */
    public static function get_collection() {
        return new data_object_collection(get_called_class());
    }

    /**
     * Retourne la liste des objets

     * @return array
     */
    public static function get_objects() {
        $objCol = new data_object_collection(get_called_class());
        return $objCol->get_objects();
    }

}
