<?php
/*
    Copyright (c) 2002-2007 Netlor
    Copyright (c) 2007-2008 Ovensia
    Contributors hold Copyright (c) to their code submissions.

    This file is part of Ploopi.

    Ploopi is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ploopi is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ploopi; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Gestion des actions utilisateur.
 * Lecture/Ecriture dans le log des actions excutes, rcupration de la liste des utilisateurs pouvant excuter une action dtermine.
 *
 * @package ploopi
 * @subpackage action
 * @copyright Netlor, Ovensia
 * @license GNU General Public License (GPL)
 * @author Stphane Escaich
 */

/**
 * Enregistre le log d'une action utilisateur
 *
 * @param int $id_action identifiant de l'action
 * @param string $id_record identifiant de l'enregistrement
 * @param int $id_module_type identifiant du type de module
 * @param int $id_module identifiant du module
 */

function ploopi_create_user_action_log($id_action, $id_record, $id_module_type = 0, $id_module = 0, $id_workspace = 0)
{
    include_once './include/classes/log.php';
    include_once './include/classes/mb.php';

    global $db;

    $user_action_log = new user_action_log();
    $user_action_log->fields['user'] = '';
    $user_action_log->fields['workspace'] = '';
    $user_action_log->fields['module'] = '';
    $user_action_log->fields['module_type'] = '';
    $user_action_log->fields['action'] = '';
    $user_action_log->fields['id_user'] = 0;
    $user_action_log->fields['id_workspace'] = $id_workspace;
    $user_action_log->fields['id_module_type'] = $id_module_type;
    $user_action_log->fields['id_module'] = $id_module;
    $user_action_log->fields['id_action'] = $id_action;
    $user_action_log->fields['id_record'] = $id_record;

    if (isset($_SESSION)) {
        if ($id_module_type == 0) $id_module_type = $_SESSION['ploopi']['moduletypeid'];
        if ($id_module == 0) $id_module = $_SESSION['ploopi']['moduleid'];
        if ($id_workspace == 0) $id_workspace = $_SESSION['ploopi']['workspaceid'];

        $user_action_log->fields['user'] = isset($_SESSION['ploopi']['user']) ? trim("{$_SESSION['ploopi']['user']['firstname']} {$_SESSION['ploopi']['user']['lastname']} ({$_SESSION['ploopi']['user']['login']})") : '';
        $user_action_log->fields['workspace'] = isset($_SESSION['ploopi']['workspaces'][$_SESSION['ploopi']['workspaceid']]) ? $_SESSION['ploopi']['workspaces'][$_SESSION['ploopi']['workspaceid']]['label'] : '';

        if (isset($_SESSION['ploopi']['modules'][$id_module]))
        {
            $user_action_log->fields['module'] = $_SESSION['ploopi']['modules'][$id_module]['label'];
            $user_action_log->fields['module_type'] = $_SESSION['ploopi']['modules'][$id_module]['moduletype'];
        }

        $user_action_log->fields['id_user'] = isset($_SESSION['ploopi']['userid']) ? $_SESSION['ploopi']['userid'] : 0;
        $user_action_log->fields['id_workspace'] = $id_workspace;
    }
    else {
        include_once './include/classes/module.php';
        include_once './include/classes/workspace.php';

        $module = new module();
        if ($module->open($id_module)) $user_action_log->fields['module'] = $module->fields['label'];

        $module_type = new module_type();
        if ($module_type->open($id_module_type)) $user_action_log->fields['module_type'] = $module_type->fields['label'];

        $workspace = new workspace();
        if ($workspace->open($id_workspace)) $user_action_log->fields['workspace'] = $workspace->fields['label'];
    }

    $action = new mb_action();
    if ($action->open($id_module_type, $id_action)) $user_action_log->fields['action'] = $action->fields['label'];
    else $user_action_log->fields['action'] = '';

    $user_action_log->fields['ip'] = (empty($_SESSION['ploopi']['remote_ip'])) ? php_sapi_name() : implode(',', $_SESSION['ploopi']['remote_ip']);
    $user_action_log->fields['timestp'] = ploopi_createtimestamp();
    $user_action_log->save();
}

/**
 * Renvoie le log pour une action ou un objet
 *
 * @param string $id_record identifiant de l'enregistrement
 * @param int $id_object identifiant de l'objet
 * @param int $id_action identifiant de l'action
 * @param int $id_module_type identifiant du type de module
 * @param int $id_module identifiant du module
 * @param int $limit_offset valeur infrieur de la clause LIMIT
 * @param int $limit_count nombre de lignes de log  renvoyer
 * @return array tableau contenant la liste des actions
 */

function ploopi_get_user_action_log($id_record, $id_object = -1, $id_action = -1, $id_module_type = -1, $id_module = -1, $limit_offset = 0, $limit_count = 25)
{
    global $db;

    if ($id_module_type == -1) $id_module_type = $_SESSION['ploopi']['moduletypeid'];
    if ($id_module == -1) $id_module = $_SESSION['ploopi']['moduleid'];

    $where = '';
    if ($id_action != -1) $where .= " AND ploopi_user_action_log.id_action = {$id_action}";
    if ($id_object != -1) $where .= " AND ploopi_mb_action.id_object = {$id_object}";

    $sql =  "
            SELECT      ploopi_user_action_log.*,
                        ploopi_user.id,
                        ploopi_user.lastname,
                        ploopi_user.firstname,
                        ploopi_mb_action.label

            FROM        ploopi_user_action_log

            INNER JOIN  ploopi_mb_action
            ON          ploopi_mb_action.id_action = ploopi_user_action_log.id_action
            AND         ploopi_mb_action.id_module_type = ploopi_user_action_log.id_module_type

            LEFT JOIN   ploopi_user ON ploopi_user.id = ploopi_user_action_log.id_user
            WHERE       ploopi_user_action_log.id_module_type = $id_module_type
            AND         ploopi_user_action_log.id_module = $id_module
            AND         ploopi_user_action_log.id_record = $id_record
            {$where}
            ORDER BY    timestp DESC
            LIMIT {$limit_offset}, {$limit_count}
            ";

    $result = $db->query($sql);
    $user_action = array();
    while ($fields = $db->fetchrow($result))
    {
        $user_action[] = array(
                                    'timestp'   =>  $fields['timestp'],
                                    'id_action' =>  $fields['id_action'],
                                    'action_label'  =>  $fields['label'],
                                    'id_user'   =>  $fields['id_user'],
                                    'user_name' =>  ($fields['id'] == null) ? _PLOOPI_UNKNOWNUSER : "{$fields['lastname']} {$fields['firstname']}"
                                );
    }

    return($user_action);
}

/**
 * Renvoie un tableau des utilisateurs qui peuvent excuter une action
 *
 * @param mixed $id_action identifiant de l'action
 * @param integer $id_module identifiant du module (optionnel)
 * @param mixed $id_workspace identifiant de l'espace (optionnel)
 * @return array tableau des utilisateurs (tableau index d'id)
 */

function ploopi_actions_getusers($id_action, $id_module = -1, $id_workspace = -1)
{
    global $db;

    if ($id_module == -1) $id_module = $_SESSION['ploopi']['moduleid'];
    if ($id_workspace == -1) $id_workspace = $_SESSION['ploopi']['workspaceid'];
    if (!is_array($id_action)) $id_action = array($id_action);
    if (!is_array($id_workspace)) $id_workspace = array($id_workspace);

    $id_action = array_map('intval', $id_action);
    $id_module = intval($id_module);
    $id_workspace = array_map('intval', $id_workspace);

    $rs = $db->query($sql = "
        SELECT *

        FROM (
            SELECT      u.*, wu.id_workspace
            FROM        ploopi_user u
            INNER JOIN  ploopi_workspace_user wu ON wu.id_user = u.id AND wu.id_workspace IN (".implode(',', $id_workspace).")
            LEFT JOIN   ploopi_workspace_user_role wur ON wur.id_user = u.id AND wur.id_workspace = wu.id_workspace
            LEFT JOIN   ploopi_role r ON r.id = wur.id_role AND r.id_module = {$id_module}
            LEFT JOIN   ploopi_role_action ra ON ra.id_role = r.id AND ra.id_action IN (".implode(',', $id_action).")
            WHERE       wu.adminlevel = "._PLOOPI_ID_LEVEL_SYSTEMADMIN." OR NOT ISNULL(ra.id_role)

            UNION

            SELECT      u.*, wg.id_workspace
            FROM        ploopi_user u
            INNER JOIN  ploopi_group_user gu ON gu.id_user = u.id
            INNER JOIN  ploopi_workspace_group wg ON wg.id_group = gu.id_group AND wg.id_workspace IN (".implode(',', $id_workspace).")
            LEFT JOIN   ploopi_workspace_group_role wgr ON wgr.id_group = gu.id_group AND wgr.id_workspace = wg.id_workspace
            LEFT JOIN   ploopi_role r ON r.id = wgr.id_role AND r.id_module = {$id_module}
            LEFT JOIN   ploopi_role_action ra ON ra.id_role = r.id AND ra.id_action IN (".implode(',', $id_action).")
            WHERE       wg.adminlevel = "._PLOOPI_ID_LEVEL_SYSTEMADMIN." OR NOT ISNULL(ra.id_role)
        ) as us

        GROUP BY us.id
    ");

    return $db->getarray($rs, true);
}
